@@ -30,6 +30,13 @@ pub enum DataType {
     PathBuf,
     Count32,
     Averagef32,
+    Date,
+    DateTime,
+    Duration,
+    LocalTime,
+    UtcTime,
+    Period,
+    Timestamp,
     Object(String),
     ExternalObject(String),
     Booleans,
@@ -112,6 +119,13 @@ pub fn data_ty_to_literal(ty: &DataType) -> String {
         DataType::PathBuf => "std::path::PathBuf".to_owned(),
         DataType::Count32 => "Count32".to_owned(),
         DataType::Averagef32 => "Averagef32".to_owned(),
+        DataType::Date => "chrono::NaiveDate".to_owned(),
+        DataType::DateTime => "chrono::NaiveDateTime".to_owned(),
+        DataType::Duration => "chrono::Duration".to_owned(),
+        DataType::LocalTime => "chrono::DateTime<chrono::Local>".to_owned(),
+        DataType::UtcTime => "chrono::DateTime<chrono::Utc>".to_owned(),
+        DataType::Period => "pipebase::common::Period".to_owned(),
+        DataType::Timestamp => "pipebase::common::Timestamp".to_owned(),
         DataType::Object(object) => object.to_owned(),
         DataType::ExternalObject(object) => object.to_owned(),
         DataType::Booleans => {
